@@ -28,13 +28,13 @@ int _gravatar_log_dom = -1;
 
 typedef struct _Gravatar {
      const char *url;
-     Config *config;
+     const Config *config;
 } Gravatar;
 
 static Evas_Object *
 _tooltip_content(void *data,
                  Evas_Object *obj,
-                 Evas_Object *tt EINA_UNUSED)
+                 Evas_Object *_tt EINA_UNUSED)
 {
    Gravatar *g = data;
    Evas_Object *o;
@@ -47,8 +47,8 @@ _tooltip_content(void *data,
 
 static void
 _tooltip_del(void            *data,
-             Evas_Object *obj EINA_UNUSED,
-             void            *event_info EINA_UNUSED)
+             Evas_Object     *_obj EINA_UNUSED,
+             void            *_event_info EINA_UNUSED)
 {
    Gravatar *g = data;
    eina_stringshare_del(g->url);
@@ -56,7 +56,7 @@ _tooltip_del(void            *data,
 }
 
 void
-gravatar_tooltip(Evas_Object *obj, Config *config, char *email)
+gravatar_tooltip(Evas_Object *obj, const Config *config, char *email)
 {
    int n;
    MD5_CTX ctx;
