@@ -128,10 +128,14 @@ _parse_font_name(const char *full_name,
 }
 
 static void
-_cb_op_font_sel(void *data, Evas_Object *obj EINA_UNUSED, void *event EINA_UNUSED)
+_cb_op_font_sel(void *data,
+                Evas_Object *_obj EINA_UNUSED,
+                void *_event EINA_UNUSED)
 {
    Font *f = data;
    Config *config = termio_config_get(f->term);
+   Term *term = termio_term_get(f->term);
+
    if ((config->font.name) && (!strcmp(f->full_name, config->font.name)))
      return;
    if (config->font.name) eina_stringshare_del(config->font.name);
@@ -142,20 +146,25 @@ _cb_op_font_sel(void *data, Evas_Object *obj EINA_UNUSED, void *event EINA_UNUSE
    elm_object_disabled_set(op_fontslider, f->bitmap);
    elm_object_disabled_set(op_fbig, f->bitmap);
    config_save(config, NULL);
+   win_font_update(term);
 }
 
 static void
-_cb_op_fontsize_sel(void *data, Evas_Object *obj, void *event EINA_UNUSED)
+_cb_op_fontsize_sel(void *data,
+                    Evas_Object *obj,
+                    void *_event EINA_UNUSED)
 {
-   Evas_Object *term = data;
-   Config *config = termio_config_get(term);
+   Evas_Object *termio_obj = data;
+   Config *config = termio_config_get(termio_obj);
+   Term *term = termio_term_get(termio_obj);
    int size = elm_slider_value_get(obj) + 0.5;
 
    if (config->font.size == size) return;
    config->font.size = size;
-   _update_sizing(term);
+   _update_sizing(termio_obj);
    elm_genlist_realized_items_update(op_fontlist);
    config_save(config, NULL);
+   win_font_update(term);
 }
 
 static int
@@ -165,7 +174,10 @@ _cb_op_font_sort(const void *d1, const void *d2)
 }
 
 static void
-_cb_op_font_preview_del(void *data EINA_UNUSED, Evas *e EINA_UNUSED, Evas_Object *obj, void *event EINA_UNUSED)
+_cb_op_font_preview_del(void *_data EINA_UNUSED,
+                        Evas *_e EINA_UNUSED,
+                        Evas_Object *obj,
+                        void *_event EINA_UNUSED)
 {
    Evas_Object *o;
    Ecore_Timer *timer = evas_object_data_get(obj, "delay");
@@ -233,7 +245,10 @@ done:
 }
 
 static void
-_cb_op_font_preview_eval(void *data, Evas *e EINA_UNUSED, Evas_Object *obj, void *event EINA_UNUSED)
+_cb_op_font_preview_eval(void *data,
+                         Evas *_e EINA_UNUSED,
+                         Evas_Object *obj,
+                         void *_event EINA_UNUSED)
 {
    Font *f = data;
    Evas_Coord ox, oy, ow, oh, vx, vy, vw, vh;
@@ -287,20 +302,27 @@ _cb_op_font_content_get(void *data, Evas_Object *obj, const char *part)
 }
 
 static char *
-_cb_op_font_text_get(void *data, Evas_Object *obj EINA_UNUSED, const char *part EINA_UNUSED)
+_cb_op_font_text_get(void *data,
+                     Evas_Object *_obj EINA_UNUSED,
+                     const char *_part EINA_UNUSED)
 {
    Font *f = data;
    return strdup(f->pretty_name);
 }
 
 static char *
-_cb_op_font_group_text_get(void *data, Evas_Object *obj EINA_UNUSED, const char *part EINA_UNUSED)
+_cb_op_font_group_text_get(void *data,
+                           Evas_Object *_obj EINA_UNUSED,
+                           const char *_part EINA_UNUSED)
 {
    return strdup(data);
 }
 
 static void
-_cb_term_resize(void *data, Evas *e EINA_UNUSED, Evas_Object *obj EINA_UNUSED, void *event EINA_UNUSED)
+_cb_term_resize(void *data,
+                Evas *_e EINA_UNUSED,
+                Evas_Object *_obj EINA_UNUSED,
+                void *_event EINA_UNUSED)
 {
    Evas_Object *term = data;
    if (expecting_resize)
@@ -312,7 +334,10 @@ _cb_term_resize(void *data, Evas *e EINA_UNUSED, Evas_Object *obj EINA_UNUSED, v
 }
 
 static void
-_cb_font_del(void *data, Evas *e EINA_UNUSED, Evas_Object *obj EINA_UNUSED, void *event EINA_UNUSED)
+_cb_font_del(void *data,
+             Evas *_e EINA_UNUSED,
+             Evas_Object *_obj EINA_UNUSED,
+             void *_event EINA_UNUSED)
 {
    Evas_Object *term = data;
    evas_object_event_callback_del_full(term, EVAS_CALLBACK_RESIZE, 
@@ -342,6 +367,19 @@ options_font_clear(void)
      }
 }
 
+static void
+_cb_font_bolditalic(void *data,
+                    Evas_Object *obj,
+                    void *_event EINA_UNUSED)
+{
+   Evas_Object *term = data;
+   Config *config = termio_config_get(term);
+
+   config->font.bolditalic = elm_check_state_get(obj);
+   termio_config_update(term);
+   config_save(config, NULL);
+}
+
 void
 options_font(Evas_Object *opbox, Evas_Object *term)
 {
@@ -433,6 +471,7 @@ options_font(Evas_Object *opbox, Evas_Object *term)
      {
         char *s;
         f = calloc(1, sizeof(Font));
+        if (!f) break;
         f->full_name = eina_stringshare_add(file);
         s = strchr(file, '.');
         if (s != NULL) *s = '\0';
@@ -474,6 +513,7 @@ options_font(Evas_Object *opbox, Evas_Object *term)
         if (!eina_hash_find(fonthash, fname))
           {
              f = calloc(1, sizeof(Font));
+             if (!f) break;
              if (_parse_font_name(fname, &f->full_name, &f->pretty_name) <0)
                {
                   free(f);
@@ -520,6 +560,16 @@ options_font(Evas_Object *opbox, Evas_Object *term)
    evas_object_size_hint_align_set(opbox, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_show(o);
 
+   o = elm_check_add(bx0);
+   evas_object_size_hint_weight_set(o, EVAS_HINT_EXPAND, 0.0);
+   evas_object_size_hint_align_set(o, EVAS_HINT_FILL, 0.5);
+   elm_object_text_set(o, _("Display bold and italic in the terminal"));
+   elm_check_state_set(o, config->font.bolditalic);
+   elm_box_pack_end(bx0, o);
+   evas_object_show(o);
+   evas_object_smart_callback_add(o, "changed",
+                                  _cb_font_bolditalic, term);
+
    expecting_resize = 0;
    evas_object_geometry_get(term, NULL, NULL, &tsize_w, &tsize_h);
    evas_object_event_callback_add(term, EVAS_CALLBACK_RESIZE,
