@@ -1,29 +1,36 @@
+#include "private.h"
 #include <stdio.h>
 #include <limits.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
 
+#include <Eina.h>
+#include "tycommon.h"
+
+static void
+print_usage(const char *argv0)
+{
+   printf("Usage: %s "HELP_ARGUMENT_SHORT" [-p] [FILE1 FILE2 ...]\n"
+          "  Change the terminal background to the given file/uri\n"
+          "  -p  Make change permanent (stored in config)\n"
+          HELP_ARGUMENT_DOC"\n"
+          "\n",
+          argv0);
+}
+
 int
 main(int argc, char **argv)
 {
    int i, perm = 0;
-   
-   if (!getenv("TERMINOLOGY")) return 0;
-   if (argc > 1 &&
-       (!strcmp(argv[1], "-h") || !strcmp(argv[1], "--help")))
-     {
-        printf("Usage: %s [-p] [FILE1 FILE2 ...]\n"
-               "  Change the terminal background to the given file/uri\n"
-               "  -p  Make change permanent (stored in config)\n"
-               "\n",
-               argv[0]);
-        return 0;
-     }
+
+   ON_NOT_RUNNING_IN_TERMINOLOGY_EXIT_1();
+   ARGUMENT_ENTRY_CHECK(argc, argv, print_usage);
+
    if (argc <= 1)
      {
         char tbuf[32];
-          snprintf(tbuf, sizeof(tbuf), "%c}bt", 0x1b);
+        snprintf(tbuf, sizeof(tbuf), "%c}bt", 0x1b);
         if (write(0, tbuf, strlen(tbuf) + 1) != (signed)(strlen(tbuf) + 1)) perror("write");
         return 0;
      }
