@@ -1,3 +1,4 @@
+#include "private.h"
 #include <Eina.h>
 #include <Ecore.h>
 #include <Evas.h>
@@ -11,6 +12,8 @@
 #include <unistd.h>
 #include <string.h>
 
+#include "tycommon.h"
+
 enum {
   CENTER,
   FILL,
@@ -118,12 +121,13 @@ prnt(const char *path, int w, int h, int mode)
 static void
 print_usage(const char *argv0)
 {
-   printf("Usage: %s [-s|-f|-c] [-g <width>x<height>] FILE1 [FILE2 ...]\n"
+   printf("Usage: %s "HELP_ARGUMENT_SHORT" [-s|-f|-c] [-g <width>x<height>] FILE1 [FILE2 ...]\n"
           "\n"
           "  -s  Stretch file to fill nearest character cell size\n"
           "  -f  Fill file to totally cover character cells with no gaps\n"
           "  -c  Center file in nearest character cells but only scale down (default)\n"
-          "  -g <width>x<height>  Set maximum geometry for the image (cell count)\n",
+          "  -g <width>x<height>  Set maximum geometry for the image (cell count)\n"
+          HELP_ARGUMENT_DOC"\n",
          argv0);
 }
 
@@ -308,7 +312,8 @@ main(int argc, char **argv)
    char *rp;
    Eina_List *file_q = NULL;
 
-   if (!getenv("TERMINOLOGY")) return 0;
+   ON_NOT_RUNNING_IN_TERMINOLOGY_EXIT_1();
+   ARGUMENT_ENTRY_CHECK(argc, argv, print_usage);
    if (argc <= 1)
      {
         print_usage(argv[0]);
